/* A Bison parser, made by GNU Bison 3.8.2.  */

/* Bison interface for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

#ifndef YY_YY_CTF_PARSER_H_INCLUDED
# define YY_YY_CTF_PARSER_H_INCLUDED
/* Debug traces.  */
#ifndef YYDEBUG
# define YYDEBUG 1
#endif
#if YYDEBUG
extern int yydebug;
#endif

/* Token kinds.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
  enum yytokentype
  {
    YYEMPTY = -2,
    YYEOF = 0,                     /* "end of file"  */
    YYerror = 256,                 /* error  */
    YYUNDEF = 257,                 /* "invalid token"  */
    INTEGER_LITERAL = 258,         /* INTEGER_LITERAL  */
    STRING_LITERAL = 259,          /* STRING_LITERAL  */
    CHARACTER_LITERAL = 260,       /* CHARACTER_LITERAL  */
    LSBRAC = 261,                  /* LSBRAC  */
    RSBRAC = 262,                  /* RSBRAC  */
    LPAREN = 263,                  /* LPAREN  */
    RPAREN = 264,                  /* RPAREN  */
    LBRAC = 265,                   /* LBRAC  */
    RBRAC = 266,                   /* RBRAC  */
    RARROW = 267,                  /* RARROW  */
    STAR = 268,                    /* STAR  */
    PLUS = 269,                    /* PLUS  */
    MINUS = 270,                   /* MINUS  */
    LT = 271,                      /* LT  */
    GT = 272,                      /* GT  */
    TYPEASSIGN = 273,              /* TYPEASSIGN  */
    COLON = 274,                   /* COLON  */
    SEMICOLON = 275,               /* SEMICOLON  */
    DOTDOTDOT = 276,               /* DOTDOTDOT  */
    DOT = 277,                     /* DOT  */
    EQUAL = 278,                   /* EQUAL  */
    COMMA = 279,                   /* COMMA  */
    CONST = 280,                   /* CONST  */
    CHAR = 281,                    /* CHAR  */
    DOUBLE = 282,                  /* DOUBLE  */
    ENUM = 283,                    /* ENUM  */
    ENV = 284,                     /* ENV  */
    EVENT = 285,                   /* EVENT  */
    FLOATING_POINT = 286,          /* FLOATING_POINT  */
    FLOAT = 287,                   /* FLOAT  */
    INTEGER = 288,                 /* INTEGER  */
    INT = 289,                     /* INT  */
    LONG = 290,                    /* LONG  */
    SHORT = 291,                   /* SHORT  */
    SIGNED = 292,                  /* SIGNED  */
    STREAM = 293,                  /* STREAM  */
    STRING = 294,                  /* STRING  */
    STRUCT = 295,                  /* STRUCT  */
    TRACE = 296,                   /* TRACE  */
    CALLSITE = 297,                /* CALLSITE  */
    CLOCK = 298,                   /* CLOCK  */
    TYPEALIAS = 299,               /* TYPEALIAS  */
    TYPEDEF = 300,                 /* TYPEDEF  */
    UNSIGNED = 301,                /* UNSIGNED  */
    VARIANT = 302,                 /* VARIANT  */
    VOID = 303,                    /* VOID  */
    _BOOL = 304,                   /* _BOOL  */
    _COMPLEX = 305,                /* _COMPLEX  */
    _IMAGINARY = 306,              /* _IMAGINARY  */
    TOK_ALIGN = 307,               /* TOK_ALIGN  */
    IDENTIFIER = 308,              /* IDENTIFIER  */
    ID_TYPE = 309,                 /* ID_TYPE  */
    ERROR = 310                    /* ERROR  */
  };
  typedef enum yytokentype yytoken_kind_t;
#endif
/* Token kinds.  */
#define YYEMPTY -2
#define YYEOF 0
#define YYerror 256
#define YYUNDEF 257
#define INTEGER_LITERAL 258
#define STRING_LITERAL 259
#define CHARACTER_LITERAL 260
#define LSBRAC 261
#define RSBRAC 262
#define LPAREN 263
#define RPAREN 264
#define LBRAC 265
#define RBRAC 266
#define RARROW 267
#define STAR 268
#define PLUS 269
#define MINUS 270
#define LT 271
#define GT 272
#define TYPEASSIGN 273
#define COLON 274
#define SEMICOLON 275
#define DOTDOTDOT 276
#define DOT 277
#define EQUAL 278
#define COMMA 279
#define CONST 280
#define CHAR 281
#define DOUBLE 282
#define ENUM 283
#define ENV 284
#define EVENT 285
#define FLOATING_POINT 286
#define FLOAT 287
#define INTEGER 288
#define INT 289
#define LONG 290
#define SHORT 291
#define SIGNED 292
#define STREAM 293
#define STRING 294
#define STRUCT 295
#define TRACE 296
#define CALLSITE 297
#define CLOCK 298
#define TYPEALIAS 299
#define TYPEDEF 300
#define UNSIGNED 301
#define VARIANT 302
#define VOID 303
#define _BOOL 304
#define _COMPLEX 305
#define _IMAGINARY 306
#define TOK_ALIGN 307
#define IDENTIFIER 308
#define ID_TYPE 309
#define ERROR 310

/* Value type.  */
#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
union YYSTYPE
{
#line 1060 "../../../../git/formats/ctf/metadata/ctf-parser.y"

	long long ll;
	unsigned long long ull;
	char c;
	char *s;
	struct ctf_node *n;

#line 185 "ctf-parser.h"

};
typedef union YYSTYPE YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define YYSTYPE_IS_DECLARED 1
#endif




int yyparse (struct ctf_scanner *scanner, yyscan_t yyscanner);


#endif /* !YY_YY_CTF_PARSER_H_INCLUDED  */
